import { scaleLinear } from "d3-scale";
import { format } from "d3-format";
import { DEFAULT_TICKS, DEFAULT_TICK_FORMAT } from "./constants";
export class AxisTicksSetter {
    constructor(cle) {
        this.cle = cle;
    }
    /**
     * Sets the xScale property based on the value of the scaleType property
     */
    setXScale() {
        const { scaleType, marginLeft, width, marginRight } = this.cle;
        switch (scaleType) {
            case "continuous" /* Continuous */:
                this.xScale = scaleLinear()
                    .domain(this.cle.domain)
                    .range([marginLeft, width - marginRight]);
                break;
            case "discrete" /* Discrete */:
            case "threshold" /* Threshold */:
                this.xScale = scaleLinear()
                    .domain([
                    this.cle.domain[0],
                    this.cle.domain[this.cle.domain.length - 1],
                ])
                    .rangeRound([marginLeft, width - marginRight]);
                break;
            case "categorical" /* Categorical */:
                // xScale is not used for ScaleType.Categorical
                this.xScale = null;
                break;
            default:
                throw new Error(`Unrecognized scaleType: ${scaleType}`);
        }
    }
    /**
     * Handles setting fallback axis tick values for discrete & threshold scales
     * Handles setting the tickFormatter function
     */
    handleAxisTicks() {
        var _a, _b, _c;
        const { scaleType } = this.cle;
        if (scaleType !== "continuous" /* Continuous */ &&
            scaleType !== "categorical" /* Categorical */) {
            const [min, max] = this.xScale.domain();
            this.cle.tickValues = this.cle.tickValues || [
                min,
                ...(((_b = (_a = this.cle.colorScale) === null || _a === void 0 ? void 0 : _a.thresholds) === null || _b === void 0 ? void 0 : _b.call(_a)) ||
                    this.cle.colorScale.domain()),
                max,
            ];
        }
        if ((_c = this.cle.tickFormat) === null || _c === void 0 ? void 0 : _c.length) {
            this.cle.tickFormatter = format(this.cle.tickFormat);
        }
        else {
            this.cle.tickFormatter = this.xScale.tickFormat(this.cle.ticks || DEFAULT_TICKS, this.cle.tickFormat || DEFAULT_TICK_FORMAT);
        }
    }
}
//# sourceMappingURL=x-scale-axis.js.map